/*****************************************************************************
* wanpipe_tdm_api.h 
* 		
* 		WANPIPE(tm) AFT TE1 Hardware Support
*
* Authors: 	Nenad Corbic <ncorbic@sangoma.com>
*
* Copyright:	(c) 2003-2005 Sangoma Technologies Inc.
*
*		This program is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
* ============================================================================
* Oct 04, 2005	Nenad Corbic	Initial version.
*****************************************************************************/

#ifndef __WANPIPE_TDM_API_H_
#define __WANPIPE_TDM_API_H_

#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
# include <net/wanpipe_includes.h>
# include <net/wanpipe_defines.h>
# include <net/wanpipe.h>
# include <net/wanpipe_codec_iface.h>
#else
# include <linux/wanpipe_includes.h>
# include <linux/wanpipe_defines.h>
# include <linux/wanpipe.h>
# include <linux/if_wanpipe.h>
# include <linux/wanpipe_codec_iface.h>
#endif

enum wanpipe_tdm_api_cmds {

	SIOC_WP_TDM_GET_USR_MTU_MRU,

	SIOC_WP_TDM_SET_HW_PERIOD,
	SIOC_WP_TDM_GET_HW_PERIOD,
	SIOC_WP_TDM_SET_HW_MTU_MRU,
	SIOC_WP_TDM_GET_HW_MTU_MRU,

	SIOC_WP_TDM_SET_CODEC,
	SIOC_WP_TDM_GET_CODEC,

	SIOC_WP_TDM_SET_POWER_LEVEL,
	SIOC_WP_TDM_GET_POWER_LEVEL,

	SIOC_WP_TDM_TOGGLE_RX,
	SIOC_WP_TDM_TOGGLE_TX,

	SIOC_WP_TDM_GET_HW_CODING,
	SIOC_WP_TDM_SET_HW_CODING,

	SIOC_WP_TDM_GET_FULL_CFG,

	SIOC_WP_TDM_SET_EC_TAP,
	SIOC_WP_TDM_GET_EC_TAP

};

typedef struct wanpipe_tdm_api{

	unsigned int cmd;	
	unsigned int hw_tdm_coding;	/* Set/Get HW TDM coding: uLaw muLaw */
	unsigned int hw_mtu_mru;	/* Set/Get HW TDM MTU/MRU */
	unsigned int hw_period;		/* Set/Get HW Period in ms */
	unsigned int tdm_codec;		/* Set/Get TDM Codec: SLinear */
	unsigned int power_level;	/* Set/Get Power level treshold */
	unsigned int rx_disable;	/* Enable/Disable Rx */
	unsigned int tx_disable;	/* Enable/Disable Tx */		
	unsigned int usr_mtu_mru;	/* Set/Get User TDM MTU/MRU */
	unsigned int ec_tap;		/* Echo Cancellation Tap */

}wanpipe_tdm_api_t;


#ifdef WAN_KERNEL


typedef struct wanpipe_tdm_api_cfg {

	void 	*chan;
	char 	name[WAN_IFNAME_SZ];
	wan_spinlock_t lock;
	
	wanpipe_tdm_api_t	cfg;

	int (*get_hw_mtu_mru)(void *chan);
	int (*set_hw_mtu_mru)(void *chan, u32 hw_mtu_mru);
	int (*get_usr_mtu_mru)(void *chan);
	int (*set_usr_mtu_mru)(void *chan, u32 usr_mtu_mru);

}wanpipe_tdm_api_cfg_t;

static __inline int is_tdm_api(void *chan, wanpipe_tdm_api_cfg_t *tdm_api)
{
	if (tdm_api->chan == chan){
		return 1;
	}
	return 0;
}

extern int wanpipe_tdm_api_ioctl(wanpipe_tdm_api_cfg_t *tdm_api, struct ifreq *ifr);

extern int wanpipe_tdm_api_tx_rx (wanpipe_tdm_api_cfg_t *tdm_api, 
				  netskb_t **skb, 
                                  u16 *power, 
				  int tx_dir);

#define wanpipe_tdm_api_tx(__tdm_api,__skb) \
		wanpipe_tdm_api_tx_rx(__tdm_api, __skb, NULL, 1)

#define wanpipe_tdm_api_rx(__tdm_api,__skb,__power)  \
		wanpipe_tdm_api_tx_rx(__tdm_api, __skb, __power, 0)

#endif

#endif
