/*****************************************************************************
* wanpipe_tdm_api.c 
* 		
* 		WANPIPE(tm) AFT TE1 Hardware Support
*
* Authors: 	Nenad Corbic <ncorbic@sangoma.com>
*
* Copyright:	(c) 2003-2005 Sangoma Technologies Inc.
*
*		This program is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
* ============================================================================
* Oct 04, 2005	Nenad Corbic	Initial version.
*****************************************************************************/

#include <linux/wanpipe_includes.h>
#include <linux/wanpipe_defines.h>
#include <linux/wanpipe.h>
#include <linux/if_wanpipe.h>
#include <linux/wanpipe_tdm_api.h>


int wanpipe_tdm_api_ioctl(wanpipe_tdm_api_cfg_t *tdm_api, struct ifreq *ifr)
{
	wanpipe_tdm_api_t usr_tdm_api;
	int err=0;
	u32 cmd;
	wanpipe_codec_ops_t *wp_codec_ops;
	wan_smp_flag_t smp_flags;

	if (!tdm_api->chan){
		DEBUG_EVENT("%s:%d Error: TDM API Not initialized! chan=NULL!\n",
			__FUNCTION__,__LINE__);
		return -EFAULT;
	}	
	
	if (!ifr){
		return -EINVAL;
	}
	
	if (WAN_COPY_FROM_USER(&usr_tdm_api,
			       ifr->ifr_data,
			       sizeof(wanpipe_tdm_api_t))){
	       return -EFAULT;
	}

	cmd=usr_tdm_api.cmd;

	DEBUG_EVENT("%s: TDM API CMD: %i\n",tdm_api->name,cmd);

	wan_spin_lock_irq(&tdm_api->lock,&smp_flags);

	switch (cmd){

	case SIOC_WP_TDM_SET_HW_MTU_MRU:
		break;

	case SIOC_WP_TDM_GET_HW_MTU_MRU:
		tdm_api->cfg.hw_mtu_mru = tdm_api->get_hw_mtu_mru(tdm_api->chan);
		usr_tdm_api.hw_mtu_mru = tdm_api->cfg.hw_mtu_mru;
		break;
	
	case SIOC_WP_TDM_SET_HW_PERIOD:

		switch (usr_tdm_api.hw_period){

		case 10:
		case 20:
		case 30:
		case 40:
		case 50:
			usr_tdm_api.hw_mtu_mru = usr_tdm_api.hw_period*8;
			break;

		default:
			err = -EINVAL;
			DEBUG_EVENT("%s: TDM API: Invalid HW Period %i!\n",
					tdm_api->name,usr_tdm_api.hw_period);
			goto tdm_api_exit;
		}

		tdm_api->cfg.hw_mtu_mru = tdm_api->get_hw_mtu_mru(tdm_api->chan);

		if (tdm_api->cfg.hw_mtu_mru == usr_tdm_api.hw_mtu_mru){
			err = 0;
			goto tdm_api_exit;
		}

		err=tdm_api->set_hw_mtu_mru(tdm_api->chan, usr_tdm_api.hw_mtu_mru);
		if (err < 0){
			goto tdm_api_exit;
		}
		
		usr_tdm_api.usr_mtu_mru = wanpipe_codec_calc_new_mtu(tdm_api->cfg.tdm_codec,
			              				     usr_tdm_api.hw_mtu_mru);

		err=tdm_api->set_usr_mtu_mru(tdm_api->chan,usr_tdm_api.usr_mtu_mru);
		if (err < 0){
			tdm_api->cfg.usr_mtu_mru = tdm_api->get_usr_mtu_mru(tdm_api->chan);
			goto tdm_api_exit;
		}
		tdm_api->cfg.usr_mtu_mru = err;
		
		break;

	case SIOC_WP_TDM_GET_HW_PERIOD:

		tdm_api->cfg.hw_mtu_mru = tdm_api->get_hw_mtu_mru(tdm_api->chan);
		tdm_api->cfg.hw_period = tdm_api->cfg.hw_mtu_mru/8;
		usr_tdm_api.hw_period = tdm_api->cfg.hw_period;
		break;

	case SIOC_WP_TDM_GET_USR_MTU_MRU:

		tdm_api->cfg.usr_mtu_mru = tdm_api->get_usr_mtu_mru(tdm_api->chan);
		usr_tdm_api.usr_mtu_mru = tdm_api->cfg.usr_mtu_mru;
		break;

	case SIOC_WP_TDM_SET_CODEC:
		
		if (usr_tdm_api.tdm_codec == tdm_api->cfg.tdm_codec){
			err = 0;
			goto tdm_api_exit;
		}

		if (usr_tdm_api.tdm_codec < 0 || usr_tdm_api.tdm_codec >= WP_TDM_CODEC_MAX){
			err = -EINVAL;
			goto tdm_api_exit;
		} 

		if (usr_tdm_api.tdm_codec != WP_NONE){
			wp_codec_ops = WANPIPE_CODEC_OPS[tdm_api->cfg.hw_tdm_coding][usr_tdm_api.tdm_codec];
			if (!wp_codec_ops || !wp_codec_ops->init){

				err = -EINVAL;
				DEBUG_EVENT("%s: TDM API: Codec %i is unsupported!\n",
						tdm_api->name,usr_tdm_api.tdm_codec);
				goto tdm_api_exit;
			}
		}


		tdm_api->cfg.hw_mtu_mru = tdm_api->get_hw_mtu_mru(tdm_api->chan);

		tdm_api->cfg.usr_mtu_mru = wanpipe_codec_calc_new_mtu(usr_tdm_api.tdm_codec,
								      tdm_api->cfg.hw_mtu_mru);

		err=tdm_api->set_usr_mtu_mru(tdm_api->chan,tdm_api->cfg.usr_mtu_mru);
		if (err < 0){
			tdm_api->cfg.usr_mtu_mru = tdm_api->get_usr_mtu_mru(tdm_api->chan);
			goto tdm_api_exit;
		}
		tdm_api->cfg.usr_mtu_mru=err;

		tdm_api->cfg.tdm_codec = usr_tdm_api.tdm_codec;

		break;

	case SIOC_WP_TDM_GET_CODEC:
		usr_tdm_api.tdm_codec = tdm_api->cfg.tdm_codec;
		break;

	case SIOC_WP_TDM_SET_POWER_LEVEL:
		tdm_api->cfg.power_level = usr_tdm_api.power_level;
		break;

	case SIOC_WP_TDM_GET_POWER_LEVEL:
		usr_tdm_api.power_level = tdm_api->cfg.power_level;
		break;

	case SIOC_WP_TDM_TOGGLE_RX:
		if (tdm_api->cfg.tx_disable){
			tdm_api->cfg.rx_disable=0;
		}else{
			tdm_api->cfg.rx_disable=1;
		}
		break;

	case SIOC_WP_TDM_TOGGLE_TX:
		if (tdm_api->cfg.tx_disable){
			tdm_api->cfg.tx_disable=0;
		}else{
			tdm_api->cfg.tx_disable=1;
		}
		break;


	case SIOC_WP_TDM_SET_EC_TAP:

		switch (usr_tdm_api.ec_tap){
			case 0:
			case 32:
			case 64:
			case 128:
				tdm_api->cfg.ec_tap = usr_tdm_api.ec_tap;
				break;

			default:
				DEBUG_EVENT("%s: Illegal Echo Cancellation Tap \n",
						tdm_api->name);	
				err = -EINVAL;
				goto tdm_api_exit;
		}		 
	
		break;

	case SIOC_WP_TDM_GET_EC_TAP:
			
		usr_tdm_api.ec_tap = tdm_api->cfg.ec_tap;
		break;

	case SIOC_WP_TDM_GET_FULL_CFG:
		tdm_api->cfg.hw_mtu_mru = tdm_api->get_hw_mtu_mru(tdm_api->chan);
		tdm_api->cfg.hw_mtu_mru = tdm_api->get_hw_mtu_mru(tdm_api->chan);
		tdm_api->cfg.hw_period  = tdm_api->cfg.hw_mtu_mru/8;
		tdm_api->cfg.usr_mtu_mru = tdm_api->get_usr_mtu_mru(tdm_api->chan);
		
		memcpy(&usr_tdm_api,&tdm_api->cfg,sizeof(wanpipe_tdm_api_t));
		break;

	default:
		DEBUG_EVENT("%s: Invalid TDM API CMD %i\n", tdm_api->name,cmd);
		break;
	}

tdm_api_exit:
	wan_spin_unlock_irq(&tdm_api->lock,&smp_flags);
	
	if (WAN_COPY_TO_USER(ifr->ifr_data,
			     &usr_tdm_api,
			     sizeof(wanpipe_tdm_api_t))){
	       return -EFAULT;
	}
	

	return err;
}



int wanpipe_tdm_api_tx_rx (wanpipe_tdm_api_cfg_t *tdm_api, netskb_t **skb, 
                                  u16 *power, int tx_dir)
{
	netskb_t *nskb;	
	int err;
	wanpipe_codec_ops_t *wp_codec_ops;
	wan_smp_flag_t smp_flags;

	if (!tdm_api->chan){
		return 0;
	}

	if (tdm_api->cfg.tdm_codec == WP_NONE){
		return 0;
	}

	wan_spin_lock_irq(&tdm_api->lock,&smp_flags);

	wp_codec_ops=WANPIPE_CODEC_OPS[tdm_api->cfg.hw_tdm_coding][tdm_api->cfg.tdm_codec];
	if (!wp_codec_ops || !wp_codec_ops->init){
		wan_spin_unlock_irq(&tdm_api->lock,&smp_flags);
		return -EINVAL;		
	}

	wan_spin_unlock_irq(&tdm_api->lock,&smp_flags);

	if (tx_dir) {

#if 0
		if (tdm_api->ec_eng.ec){
			wp_wanpipe_ec_tx(&tdm_api->ec_eng,skb);
		}
#endif
		err = wp_codec_ops->decode(*skb,&nskb);
		if (err){
			return err;
		}

		if (wan_skb_len(nskb) != tdm_api->cfg.hw_mtu_mru){
			wan_skb_free(nskb);
			return -EINVAL;
		}

	}else{

		err = wp_codec_ops->encode(*skb,&nskb,power);	
		if (err){
			return err;
		}

	}

	wan_skb_free(*skb);
	
	*skb = nskb;

	return 0;
}


